@@ -14,6 +14,7 @@ from tg.configuration import Bunch
 
 from webhelpers.html import literal
 
+
 class MissingRendererError(Exception):
     def __init__(self, template_engine):
         Exception.__init__(self,
@@ -87,52 +88,48 @@ def _get_tg_vars():
     except AttributeError, ImportError:
         h = Bunch()
 
-
     # TODO: Implement user_agent and other missing features.
     tg_vars = Bunch(
-        config = tg.config,
-        flash_obj = tg.flash,
-        flash = DeprecatedFlashVariable(
+        config=tg.config,
+        flash_obj=tg.flash,
+        flash=DeprecatedFlashVariable(
             lambda: tg.flash.message,
             "flash is deprecated, please use flash_obj.message instead "
             "or use the new flash_obj.render() method"
             ),
-        flash_status = DeprecatedFlashVariable(
+        flash_status=DeprecatedFlashVariable(
             lambda: 'status_' + tg.flash.status,
             "flash_status is deprecated, please use flash_obj.status instead "
             "or use the new flash_obj.render() method"
             ),
-        quote_plus = quote_plus,
-        url = tg.url,
+        quote_plus=quote_plus,
+        url=tg.url,
         # this will be None if no identity
-        identity = req.environ.get('repoze.who.identity'),
-        session = tg.session,
-        locale = req.accept_language.best_matches(),
-        errors = getattr(tmpl_context, "form_errors", {}),
-        inputs = getattr(tmpl_context, "form_values", {}),
-        request = req,
-        auth_stack_enabled = 'repoze.who.plugins' in req.environ,
-        predicates = predicates,
-        )
-
+        identity=req.environ.get('repoze.who.identity'),
+        session=tg.session,
+        locale=req.accept_language.best_matches(),
+        errors=getattr(tmpl_context, "form_errors", {}),
+        inputs=getattr(tmpl_context, "form_values", {}),
+        request=req,
+        auth_stack_enabled='repoze.who.plugins' in req.environ,
+        predicates=predicates)
 
     root_vars = Bunch(
-        c = tmpl_context,
-        tmpl_context = tmpl_context,
-        response = response,
-        request = req,
+        c=tmpl_context,
+        tmpl_context=tmpl_context,
+        response=response,
+        request=req,
         config=conf,
         app_globals=app_globals,
         g=app_globals,
-        url = tg.url,
-        helpers = h,
-        h = h,
-        tg = tg_vars,
+        url=tg.url,
+        helpers=h,
+        h=h,
+        tg=tg_vars,
         translator=translator,
         ungettext=tg.i18n.ungettext,
         _=tg.i18n.ugettext,
-        N_=tg.i18n.gettext_noop,
-        )
+        N_=tg.i18n.gettext_noop)
 
     econf = conf['pylons.environ_config']
     if 'beaker.session' in req.environ or \
@@ -146,11 +143,12 @@ def _get_tg_vars():
         root_vars.update(variable_provider())
     return root_vars
 
-#Monkey patch pylons_globals for cases when pylons.templating is used
-#instead of tg.render to programmatically render templates.
+# Monkey patch pylons_globals for cases when pylons.templating is used
+# instead of tg.render to programmatically render templates.
 import pylons
 pylons.templating.pylons_globals = _get_tg_vars
-#end monkeying around
+# end monkeying around
+
 
 def render(template_vars, template_engine=None, template_name=None, **kwargs):
     config = tg.config._current_obj()
@@ -166,24 +164,34 @@ def render(template_vars, template_engine=None, template_name=None, **kwargs):
             raise MissingRendererError(template_engine)
 
     if not template_vars:
-        template_vars={}
+        template_vars = {}
 
     caching_options = template_vars.get('tg_cache', {})
     kwargs['cache_key'] = caching_options.get('key')
     kwargs['cache_expire'] = caching_options.get('expire')
-    kwargs['cache_type'] =  caching_options.get('type')
+    kwargs['cache_type'] = caching_options.get('type')
 
+    for func in config.get('hooks', {}).get('before_render_call', []):
+        func(template_engine, template_name, template_vars, kwargs)
+
+    tg_vars = template_vars
     if template_engine not in ("json", 'amf'):
         # Get the extra vars, and merge in the vars from the controller
         tg_vars = _get_tg_vars()
         tg_vars.update(template_vars)
-        template_vars = tg_vars
 
     if not render_function:
         # getting the default renderer, if no engine was defined in @expose()
-        render_function = config['render_functions'][config['default_renderer']]
+        render_function = config[
+            'render_functions'][config['default_renderer']]
+
+    kwargs['result'] = render_function(template_name, tg_vars, **kwargs)
+
+    for func in config.get('hooks', {}).get('after_render_call', []):
+        func(template_engine, template_name, template_vars, kwargs)
+
+    return kwargs['result']
 
-    return render_function(template_name, template_vars, **kwargs)
 
 def cached_template(template_name, render_func, ns_options=(),
                     cache_key=None, cache_type=None, cache_expire=None,
@@ -226,7 +234,8 @@ def cached_template(template_name, render_func, ns_options=(),
 
     """
     # If one of them is not None then the user did set something
-    if cache_key is not None or cache_expire is not None or cache_type is not None:
+    if (cache_key is not None
+            or cache_expire is not None or cache_type is not None):
         if not cache_type:
             cache_type = 'dbm'
         if not cache_key:
@@ -288,7 +297,9 @@ class RenderChameleonGenshi(object):
 class RenderGenshi(object):
     """Singleton that can be called as the Genshi render function."""
 
-    genshi_functions = {} # auxiliary Genshi functions loaded on demand
+    genshi_functions = {}  # auxiliary Genshi functions loaded on demand
+
+    default_doctype = default_method = None
 
     doctypes_for_methods = {
         'html': 'html-transitional',
@@ -296,14 +307,13 @@ class RenderGenshi(object):
 
     doctypes_for_content_type = {
         'text/html': ('html', 'html-transitional',
-        'html-frameset', 'html5',
-        'xhtml', 'xhtml-strict',
-        'xhtml-transitional', 'xhtml-frameset'),
+            'html-frameset', 'html5',
+            'xhtml', 'xhtml-strict',
+            'xhtml-transitional', 'xhtml-frameset'),
         'application/xhtml+xml': ('xhtml', 'xhtml-strict',
-        'xhtml-transitional',
-        'xhtml-frameset', 'xhtml11'),
-        'image/svg+xml': ('svg', 'svg-full', 'svg-basic', 'svg-tiny')
-        }
+            'xhtml-transitional',
+            'xhtml-frameset', 'xhtml11'),
+        'image/svg+xml': ('svg', 'svg-full', 'svg-basic', 'svg-tiny')}
 
     methods_for_content_type = {
         'text/plain': ('text',),
@@ -322,6 +332,22 @@ class RenderGenshi(object):
             from genshi import HTML, XML
             self.genshi_functions.update(HTML=HTML, XML=XML)
         self.load_template = loader.load
+        doctype = tg.config.get('templating.genshi.doctype')
+        if doctype:
+            if isinstance(doctype, str):
+                self.default_doctype = doctype
+            elif isinstance(doctype, dict):
+                doctypes = self.doctypes_for_content_type.copy()
+                doctypes.update(doctype)
+                self.doctypes_for_content_type = doctypes
+        method = tg.config.get('templating.genshi.method')
+        if method:
+            if isinstance(method, str):
+                self.default_method = method
+            elif isinstance(method, dict):
+                methods = self.methods_for_content_type.copy()
+                methods.update(method)
+                self.methods_for_content_type = methods
 
     @staticmethod
     def method_for_doctype(doctype):
@@ -338,19 +364,25 @@ class RenderGenshi(object):
         return method
 
     def __call__(self, template_name, template_vars, **kwargs):
-        """Render the template_vars with the Genshi template."""
-        config = tg.config._current_obj()
+        """Render the template_vars with the Genshi template.
+
+        If you don't pass a doctype or pass 'auto' as the doctype,
+        then the doctype will be automatically determined.
+        If you pass a doctype of None, then no doctype will be injected.
+        If you don't pass a method or pass 'auto' as the method,
+        then the method will be automatically determined.
+
+        """
         response = tg.response._current_obj()
 
         template_vars.update(self.genshi_functions)
 
         # Gets document type from content type or from config options
-        doctype = kwargs.get('doctype')
-        if not doctype:
-            doctype = config.get('templating.genshi.doctype')
+        doctype = kwargs.get('doctype', 'auto')
+        if doctype == 'auto':
+            doctype = self.default_doctype
             if not doctype:
-                method = kwargs.get('method') or config.get(
-                    'templating.genshi.method') or 'xhtml'
+                method = kwargs.get('method') or self.default_method or 'xhtml'
                 doctype = self.doctypes_for_methods.get(method)
             doctypes = self.doctypes_for_content_type.get(response.content_type)
             if doctypes and (not doctype or doctype not in doctypes):
@@ -359,8 +391,8 @@ class RenderGenshi(object):
 
         # Gets rendering method from content type or from config options
         method = kwargs.get('method')
-        if not method:
-            method = config.get('templating.genshi.method')
+        if not method or method == 'auto':
+            method = self.default_method
             if not method:
                 method = self.method_for_doctype(doctype)
             methods = self.methods_for_content_type.get(response.content_type)
@@ -377,13 +409,14 @@ class RenderGenshi(object):
                                ns_options=('doctype', 'method'), **kwargs)
 
 
-def render_mako(template_name, globs, cache_key=None, cache_type=None, cache_expire=None):
+def render_mako(template_name, globs,
+        cache_key=None, cache_type=None, cache_expire=None):
     config = tg.config._current_obj()
 
     if asbool(config.get('use_dotted_templatenames', 'true')):
-        template_name = globs['app_globals'].\
-            dotted_filename_finder.get_dotted_filename(template_name, template_extension='.mak')
-
+        template_name = globs[
+            'app_globals'].dotted_filename_finder.get_dotted_filename(
+                template_name, template_extension='.mak')
 
     # Create a render callable for the cache function
     def render_template():
@@ -394,6 +427,7 @@ def render_mako(template_name, globs, cache_key=None, cache_type=None, cache_exp
     return cached_template(template_name, render_template, cache_key=cache_key,
                            cache_type=cache_type, cache_expire=cache_expire)
 
+
 def render_json(template_name, template_vars, **kwargs):
     return tg.json_encode(template_vars)
 
@@ -413,9 +447,10 @@ def render_kajiki(template_name, globs, cache_key=None,
         template = globs['app_globals'].kajiki_loader.load(template_name)
         return literal(template(globs).render())
 
-    return cached_template(template_name, render_template, cache_key=cache_key,
-                           cache_type=cache_type, cache_expire=cache_expire,
-                           ns_options=('method'), method=method)
+    return cached_template(template_name, render_template,
+        cache_key=cache_key, cache_type=cache_type, cache_expire=cache_expire,
+        ns_options=('method'), method=method)
+
 
 def render_jinja(template_name, globs, cache_key=None,
                  cache_type=None, cache_expire=None):
@@ -428,10 +463,9 @@ def render_jinja(template_name, globs, cache_key=None,
     # Create a render callable for the cache function
     def render_template():
         # Grab a template reference
-        template = \
-            globs['app_globals'].jinja2_env.get_template(template_name)
+        template = globs['app_globals'].jinja2_env.get_template(template_name)
         return literal(template.render(**globs))
 
-    return cached_template(template_name, render_template, cache_key=cache_key,
-                           cache_type=cache_type, cache_expire=cache_expire)
+    return cached_template(template_name, render_template,
+        cache_key=cache_key, cache_type=cache_type, cache_expire=cache_expire)
 
